package com.sun.jndi.ldap;

import java.util.StringTokenizer;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.Context;
import javax.naming.NamingException;
import java.util.Arrays;
import javax.naming.directory.DirContext;
import javax.naming.spi.NamingManager;
import java.util.Hashtable;
import java.util.Map;
import javax.naming.InvalidNameException;
import java.util.List;
import javax.naming.ldap.Rdn;
import javax.naming.ldap.LdapName;
import java.util.Random;

class ServiceLocator
{
    private static final String SRV_RR = "SRV";
    private static final String[] SRV_RR_ATTR;
    private static final Random random;
    
    private ServiceLocator() {
    }
    
    static String mapDnToDomainName(String name) throws InvalidNameException {
        if (name == null) {
            return null;
        }
        StringBuffer sb = new StringBuffer();
        List<Rdn> rdns = new LdapName(name).getRdns();
        for (int i = rdns.size() - 1; i >= 0; --i) {
            Rdn rdn = (Rdn)rdns.get(i);
            if (rdn.size() == 1 && "dc".equalsIgnoreCase(rdn.getType())) {
                Object value = rdn.getValue();
                if ((value instanceof String)) {
                    if (value.equals(".") || (sb.length() == 1 && sb.charAt(0) == '.')) {
                        sb.setLength(0);
                    }
                    if (sb.length() > 0) {
                        sb.append('.');
                    }
                    sb.append(value);
                }
                else {
                    sb.setLength(0);
                }
            }
            else {
                sb.setLength(0);
            }
        }
        return (sb.length() != 0) ? sb.toString() : null;
    }
    
    static String[] getLdapService(String s, Map<?, ?> t) {
        if ((t instanceof Hashtable)) {
            return getLdapService(s, (Hashtable)t);
        }
        return getLdapService(s, new Hashtable(t));
    }
    
    static String[] getLdapService(String str, Hashtable<?, ?> environment) {
        if (str == null || str.length() == 0) {
            return null;
        }
        String string = "dns:///_ldap._tcp." + str;
        String[] hostports = null;
        try {
            Context urlContext = NamingManager.getURLContext("dns", environment);
            if (!(urlContext instanceof DirContext)) {
                return null;
            }
            Attributes attributes = ((DirContext)urlContext).getAttributes(string, ServiceLocator.SRV_RR_ATTR);
            Attribute value;
            if (attributes != null && (value = attributes.get("SRV")) != null) {
                int size = value.size();
                ServiceLocator.SrvRecord[] a = new ServiceLocator.SrvRecord[size];
                int i = 0;
                int n = 0;
                while (i < size) {
                    try {
                        a[n] = new ServiceLocator.SrvRecord((String)value.get(i));
                        ++n;
                    }
                    catch (Exception ex) {}
                    ++i;
                }
                int n2 = n;
                if (n2 < size) {
                    ServiceLocator.SrvRecord[] array = new ServiceLocator.SrvRecord[n2];
                    System.arraycopy(a, 0, array, 0, n2);
                    a = array;
                }
                if (n2 > 1) {
                    Arrays.sort(a);
                }
                hostports = extractHostports(a);
            }
        }
        catch (NamingException ex2) {}
        return hostports;
    }
    
    private static String[] extractHostports(ServiceLocator.SrvRecord[] array) {
        String[] array2 = null;
        int n = 0;
        for (int i = 0; i < array.length; ++i) {
            if (array2 == null) {
                array2 = new String[array.length];
            }
            int n2 = i;
            while (i < array.length - 1 && array[i].priority == array[i + 1].priority) {
                ++i;
            }
            int n3 = i;
            for (int n4 = n3 - n2 + 1, j = 0; j < n4; ++j) {
                array2[n++] = selectHostport(array, n2, n3);
            }
        }
        return array2;
    }
    
    private static String selectHostport(ServiceLocator.SrvRecord[] array, int i, int n) {
        if (i == n) {
            return array[i].hostport;
        }
        int sum = 0;
        for (; i <= n; ++i) {
            if (array[i] != null) {
                sum += array[i].weight;
                array[i].sum = sum;
            }
        }
        String hostport = null;
        int n2 = (sum == 0) ? 0 : ServiceLocator.random.nextInt(sum + 1);
        while (i <= n) {
            if (array[i] != null && array[i].sum >= n2) {
                hostport = array[i].hostport;
                array[i] = null;
                break;
            }
            ++i;
        }
        return hostport;
    }
    
    static {
        SRV_RR_ATTR = new String[] { "SRV" };
        random = new Random();
    }
    
    static class SrvRecord implements Comparable<ServiceLocator.SrvRecord>
    {
        int priority;
        int weight;
        int sum;
        String hostport;
        
        SrvRecord(String str) throws Exception {
            StringTokenizer stringTokenizer = new StringTokenizer(str, " ");
            if (stringTokenizer.countTokens() == 4) {
                this.priority = Integer.parseInt(stringTokenizer.nextToken());
                this.weight = Integer.parseInt(stringTokenizer.nextToken());
                this.hostport = stringTokenizer.nextToken() + ":" + stringTokenizer.nextToken();
                return;
            }
            throw new IllegalArgumentException();
        }
        
        @Override
        public int compareTo(ServiceLocator.SrvRecord srvRecord) {
            if (this.priority > srvRecord.priority) {
                return 1;
            }
            if (this.priority < srvRecord.priority) {
                return -1;
            }
            if (this.weight == 0 && srvRecord.weight != 0) {
                return -1;
            }
            if (this.weight != 0 && srvRecord.weight == 0) {
                return 1;
            }
            return 0;
        }
    }
}
